/*
** Copyright (C) 2004 Level Control Systems <jaf@lcsaudio.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef RawIOThread_h
#define RawIOThread_h

#include "dataio/DataIO.h"
#include "audiomove/AudioMoveThread.h"

namespace audiomove {

/** This class knows how to read or write LCS native audio files */
class RawIOThread : public AudioMoveThread
{
public:
   /** Constructor for reading a file from disk. */
   RawIOThread(const String & readPath);

   /** Constructor for writing a file to disk. */
   RawIOThread(const String & writePath, uint8 writeNumStreams);

   /** Destructor. */
   virtual ~RawIOThread();

   virtual status_t OpenFile();

   virtual void CloseFile();
   
   /** Returns our audio format code */
   virtual const String & GetFileName() const {return _fileName;}

   /** Returns our audio format code */
   virtual uint32 GetInputFileFormat() const {return AUDIO_FORMAT_LCSRAW;}
   virtual uint32 GetOutputFileFormat() const {return AUDIO_FORMAT_LCSRAW;}

   /** Returns our audio sampling rate */
   virtual uint32 GetInputFileSampleRate() const {return WTRX_NATIVE_SAMPLES_PER_SECOND;}
   virtual uint32 GetOutputFileSampleRate() const {return WTRX_NATIVE_SAMPLES_PER_SECOND;}

   /** Returns our audio sample width */
   virtual uint32 GetInputFileSampleWidth()  const {return AUDIO_WIDTH_FLOAT;}
   virtual uint32 GetOutputFileSampleWidth() const {return AUDIO_WIDTH_FLOAT;}

   /** Returns the number of streams in our file.  Only valid when file is open. */
   virtual uint8 GetFileStreams() const {return _numStreams;}

   /** Returns the number of sample-frames in our file.  Only valid when file is open. */
   virtual uint64 GetFileFrames() const {return _numFrames;}

   virtual bool IsOutput() const {return (_numWriteStreams > 0);}

protected:
   /** Reads or writes the next buffer of audio */
   virtual ByteBufferRef ProcessBuffer(ByteBufferRef inputRef, QString & retErrStr, bool isLastBuffer);

private:
   void CloseFileAux();

   String _fileName;
   uint8 _numWriteStreams;
   DataIORef _dataIO;
   int64 _numFrames;
   uint8 _numStreams;
};

};  // end namespace audiomove

#endif
