/* This file is Copyright 2003 Level Control Systems.  See the included LICENSE.txt file for details. */  

#include <stdio.h>

#include "system/SetupSystem.h"
#include "util/NetworkUtilityFunctions.h"

USING_NAMESPACE(muscle);

// This program accepts incoming TCP connections on a given port and prints out
// any bytes it receives in Hex format.  Good for debugging clients.
int main(int argc, char ** argv) 
{
   CompleteSetupSystem css;

   if (argc < 2)
   {
      printf("Usage:  listener <port>\n");
      exit(5);
   }
   int p = atoi(argv[1]);
   int as = CreateAcceptingSocket(p);
   if (as >= 0)
   {
      LogTime(MUSCLE_LOG_INFO, "Listening for incoming TCP connections on port %i\n", p);
      while(true) 
      {
         int ss = Accept(as);
         if (ss >= 0)
         {
            char buf[4096]; Inet_NtoA(GetPeerIPAddress(ss), buf);

            LogTime(MUSCLE_LOG_INFO, "Accepted TCP connection from %s, awaiting data...\n", buf);

            while(true)
            {
               int32 ret = ReceiveData(ss, buf, sizeof(buf), true);
               if (ret >= 0)
               {
                  printf("Received:");
                  for (int32 i=0; i<ret; i++) printf(" %02x", buf[i]);
                  printf("\n");
               }
               else break;
            }
            
            LogTime(MUSCLE_LOG_ERROR, "Session socket disconnected, awaiting next connection.\n");
            CloseSocket(ss);
         }
      }
      CloseSocket(as);
   }
   else LogTime(MUSCLE_LOG_CRITICALERROR, "Couldn't bind to port %i\n", p);
   
   return 0;
}
